# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from autocomplete_light import shortcuts as al

import django
from django import forms
from django.conf import settings
from django.conf.urls import url
from django.core.urlresolvers import reverse
from django.contrib import admin
from django.contrib.auth.admin import UserAdmin, UserCreationForm
from django.utils.translation import ugettext as _
from django.contrib.auth import get_user_model
from django.contrib.auth.models import User as DjUser
from django.utils.encoding import smart_str

from jmb.jadmin import JumboModelAdmin, register_inline, AjaxInline, ConstrainedModelForm
from jmb.organization.models import (
    Contact, Organization, OrganizationGroup, OrganizationForm,
    OrganizationType, Field, ContactRole, ContactTitle, OrganizationAddress,
)
from jmb.bank.admin import BankInline, Bank

User = get_user_model()


class ContactForm(ConstrainedModelForm):
    user = forms.ModelChoiceField(
        User.objects.all(), required=False,
        widget=al.ChoiceWidget('User')
    )
    organization = forms.ModelChoiceField(
        Organization.objects.all(),
        widget=al.ChoiceWidget('Organization')
    )
    contact_groups =  forms.ModelMultipleChoiceField(
        OrganizationGroup.objects.all(), required=False,
        widget=al.MultipleChoiceWidget('OrganizationGroup',)
    )

    class Meta:
        model = Contact
        exclude = []

class ContactInline(admin.StackedInline):
    form = ContactForm
    model = Contact
    extra = 0
    exclude =  ['date_create', 'date_last_modify', 'creator', 'last_modifier', ]
    can_delete = False
    save_as = True
    normal_fieldset = (
        (_('general information'), {
            'fields': (
                'default',
                'status',
                'user',
                ('role', 'title',),
                ('last_name', 'first_name'),
                ('phone', 'fax',),
                ('mobile', 'email'),
                'photo',
                'note',
                'ordering',
                'contact_groups'
            )
        }),
    )

    def get_fieldsets(self, request, obj=None):
        self.fieldsets = self.normal_fieldset
        return self.fieldsets


class ContactAjaxInline(AjaxInline):
    model = Contact
    fk_name = 'organization'
    list_display = ('title', 'first_name', 'last_name', 'role', 'phone', 'user',
                    'email',)


class OrganizationAddressInline(admin.StackedInline):
    model = OrganizationAddress
    extra = 0
    fieldsets = (
        (_('general information'), {'fields': (
            'status', 'address_scopes', ('address_type', 'name'),
            ('number', 'zip_code'), ('locality', 'province'),
            'country', 'note'
        )}),
    )
    exclude = ['date_create', 'date_last_modify', 'creator', 'last_modifier']
    can_delete = False


class OrganizationFormClass(forms.ModelForm):
    organization_groups = forms.ModelMultipleChoiceField(
        OrganizationGroup.objects.all(), required=False,
        widget=al.MultipleChoiceWidget('OrganizationGroup',))

    class Meta:
        model = Organization
        exclude = []


class OrganizationAdmin(JumboModelAdmin):
    form = OrganizationFormClass

    class Media:
        css = {
            "all": ("organization/css/admin_organization.css",)
        }

#    form = OrganizationAdminForm
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': (
            'status',
            ('code', 'name',),
            ('organization_type', 'organization_form',),
            ('vat', 'delivery'),
            ('payment_form', 'currency'),
            ('vat_number', 'ssn'),
            ('url', 'email'),
            ('phone', 'phone2',),
            'fax',
            'organization_groups',
        )}),
        (_('Note'), {
            'fields': ('note',),
            'classes': ('collapse',)
        }),
    )
#    inlines = [ContactInline, OrganizationAddressInline, BankInline, VectorInline, CommisionInline]
    inlines = [ContactInline, OrganizationAddressInline, BankInline]
    tabs = (
        (_('Organization'), {}),
        (_('Contacts'), {'items': [ContactInline]}),
        (_('Addresses'), {'items': [OrganizationAddress]}),
        (_('Banks'), {'items': [Bank]}),
    )
    filter_horizontal = ('organization_groups', )
    list_display = (
        'get_status_icon', 'code', 'get_organization_type', 'name', 'get_phone',
        'get_phone2', 'get_fax_icon', 'get_icon_link_url', 'get_icon_link_email',
        'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'code', 'organization_type', 'name', 'phone', 'phone2', 'fax', 'url',
        'email'
    )
    advanced_search_fields = (
        ('status__in', 'organization_type__category__in',),
        ('code__icontains', 'name__icontains', 'name__istartswith'),
        ('contacts__last_name__icontains', 'contacts__first_name__icontains'),
        ('organization_type__in', 'organization_form__in',),
        ('vat__in', 'field__in',),
        ('addresses__locality__icontains',),
    )
    search_fields = ('code', 'name',)

    def get_link_url(self, obj):
        """
        Returns link to organization URL
        """
        if obj.url:
            return """<a href=%s target="_blank">%s</a>""" % (obj.url, obj.url)
        return ""
    get_link_url.short_description = _("Url")
    get_link_url.allow_tags = True

    def get_icon_link_url(self, obj):
        """
        Returns link icon to organization URL
        """
        if obj.url:
            return """
                <a href=%(href)s target="_blank">
                    <img src='%(url)sadmin/images/web.png' alt='%(title)s' title='%(title)s'/>
                </a>""" % {
                'href': obj.url,
                'url': settings.MEDIA_URL,
                'title': obj.url
            }
        return ""
    get_icon_link_url.short_description = _("www")
    get_icon_link_url.allow_tags = True

    def get_link_email(self, obj):
        """
        Returns link to organization email
        """
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

    def get_icon_link_email(self, obj):
        """
        Returns link icon to organization email
        """
        if obj.email:
            return """
                <a href=mailto:%(email)s>
                    <img src='%(url)sadmin/images/email.png' alt='%(title)s' title='%(title)s'/>
                </a>""" % {
                'email': obj.email,
                'url': settings.STATIC_URL,
                'title': obj.email
            }
        return ""
    get_icon_link_email.short_description = _("Mail")
    get_icon_link_email.allow_tags = True

    def get_link_organization_detail(self, obj):
        """
        Return link to organization detail
        """
        return """<a href='%(reverse_url)s' title='%(title)s'>%(name)s</a>""" % {
            'reverse_url': reverse('admin:organization_organization_detail', args=[obj.pk]),
            'title': _("View Detail"),
            'name': obj.name
        }
    get_link_organization_detail.short_description = _("Organization")
    get_link_organization_detail.order_admin_field = 'name'
    get_link_organization_detail.allow_tags = True

    def get_organization_type(self, result):
        """
        Returns organization type name
        """
        return "<b>%s</b>" % (result.organization_type.name,)
    get_organization_type.short_description = _("Type")
    get_organization_type.order_admin_field = 'organization_type__category__name'
    get_organization_type.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone(self, result):
            """
            Returns link to click to call phone
            """
            from asterisk.utils import click_to_call
            return click_to_call(result.phone, get_link=True)
        get_phone.short_description = _("P1")
    else:
        def get_phone(self, result):
            """
            Returns phone number
            """
            return result.phone
        get_phone.short_description = _("Phone")
    get_phone.order_admin_field = 'phone'
    get_phone.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone2(self, result):
            """
            Returns link to click to call phone 2
            """
            from asterisk.utils import click_to_call
            return click_to_call(result.phone2, get_link=True)
        get_phone2.short_description = _("P2")
    else:
        def get_phone2(self, result):
            """
            Returns phone 2 number
            """
            return result.phone2
        get_phone2.short_description = _("Phone 2")
    get_phone2.order_admin_field = 'phone2'
    get_phone2.allow_tags = True

    if 'fax' in settings.INSTALLED_APPS:
        def get_fax_icon(self, obj):
            """
            Returns icon link to fax application
            """
            if obj.fax:
                return """
                    <a href="%(reverse_url)s" title="Send Fax to %(fax)s">
                        <img src="%(url)sadmin/images/fax.png" alt="%(title)s" title="%(title)s"/>
                    </a>
                """ % {
                    'url': settings.MEDIA_URL,
                    'reverse_url': reverse('admin:fax_fax_add'),
                    'fax': obj.fax,
                    'title': obj.fax
                }
            return ""
        get_fax_icon.short_description = _("F")
    else:
        def get_fax_icon(self, obj):
            """
            Returns fax number
            """
            return obj.fax
        get_fax_icon.short_description = _("Fax")
    get_fax_icon.order_admin_field = 'fax'
    get_fax_icon.allow_tags = True


class OrganizationGroupAdmin(JumboModelAdmin):

    class Media:
        css = {
            "all": ("organization/css/admin_organization.css",)
        }

    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name', 'description')}),
    )
    list_display = (
        'get_status_icon', 'name', 'get_edit_icon', 'get_delete_icon'
    )
    ordering = ('ordering', 'name')


class OrganizationFormAdmin(JumboModelAdmin):

    class Media:
        css = {
            "all": ("organization/css/admin_organization.css",)
        }

    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general admin'), {'fields': ('status', 'code', 'name')}),
    )
    list_display = (
        'get_status_icon', 'name', 'ordering', 'get_edit_icon', 'get_delete_icon'
    )


class OrganizationTypeAdmin(JumboModelAdmin):

    class Media:
        css = {
            "all": ("organization/css/admin_organization.css",)
        }

    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': (
            'status', 'category', ('code', 'name'), 'description', 'note', 'ordering'
        )}),
    )
    list_display = (
        'get_status_icon', 'code', 'name', 'category', 'ordering',
        'get_edit_icon', 'get_delete_icon'
    )
    ordering = ('ordering', 'code', 'name')


class FieldAdmin(JumboModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name')}),
    )
    list_display = (
        'get_status_icon', 'name', 'ordering',
        'get_edit_icon', 'get_delete_icon'
    )
    ordering = ('ordering', 'name',)


class ContactRoleAdmin(JumboModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name', 'ordering')}),
    )
    list_display = ('get_status_icon', 'name', 'ordering', 'get_edit_icon', 'get_delete_icon')
    ordering = ('ordering', 'name')


class ContactTitleAdmin(JumboModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {'fields': ('status', 'name', 'abbreviation')}),
    )
    list_display = (
        'get_status_icon', 'name', 'abbreviation',
        'get_edit_icon', 'get_delete_icon'
    )


class ContactAdmin(JumboModelAdmin):
    fieldsets = (
        (_('logs admin'), {
            'fields': ('creator', 'date_create', 'last_modifier', 'date_last_modify',),
            'classes': ('collapse',)
        }),
        (_('general information'), {
            'fields': (
                'default',
                'in_addressbook',
                'status',
                'user',
                'language',
                'organization',
                ('role', 'title',),
                ('last_name', 'first_name'),
                ('phone', 'fax',),
                ('mobile', 'email'),
                'photo',
                'note',
                'ordering',
                'contact_groups',
            )
        }),
    )
    list_display = (
        'get_status_icon', 'last_name', 'first_name', 'role', 'organization',
        'get_phone', 'get_mobile', 'get_fax_icon', 'get_icon_link_email',
        'get_user_icon', 'get_edit_icon', 'get_delete_icon'
    )
    list_display_csv = (
        'get_status_display', 'last_name', 'first_name',
        'role', 'organization', 'phone', 'mobile', 'fax'
    )
    advanced_search_fields = (
        ('status__in', 'organization__organization_type__category__in'),
        ('role__in', 'last_name__icontains', 'first_name__icontains'),
        ('organization__name__icontains:organization',),
        ('email__icontains', 'user__username__icontains',),
    )
    list_filter = ('status', 'role', 'title')
    search_fields = ('last_name', 'first_name')
    ordering = ('last_name', 'first_name')

    def get_link_email(self, obj):
        """
        Returns link to contact email
        """
        if obj.email:
            return """<a href=mailto:%s>%s</a>""" % (obj.email, obj.email)
        return ""
    get_link_email.short_description = _("Email")
    get_link_email.allow_tags = True

    def get_icon_link_email(self, obj):
        """
        Returns icon link to contact email
        """
        if obj.email:
            return """
                <a href=mailto:%(email)s>
                    <img src='%(url)sjmb/images/email.png' alt='%(title)s' title='%(title)s'/>
                </a>
                """ % {
                'email': obj.email,
                'url': settings.STATIC_URL,
                'title': obj.email
            }
        return ""
    get_icon_link_email.short_description = _("Mail")
    get_icon_link_email.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_phone(self, obj):
            from asterisk.utils import click_to_call
            return click_to_call(obj.phone, get_link=True)
        get_phone.short_description = _("P")
    else:
        def get_phone(self, obj):
            return obj.phone or ''
        get_phone.short_description = _("Phone")
    get_phone.order_admin_field = 'phone'
    get_phone.allow_tags = True

    if 'asterisk' in settings.INSTALLED_APPS:
        def get_mobile(self, obj):
            from asterisk.utils import click_to_call
            return click_to_call(obj.mobile, get_link=True)
        get_mobile.short_description = _("M")
    else:
        def get_mobile(self, obj):
            return obj.mobile or ''
        get_mobile.short_description = _("Mobile")
    get_mobile.order_admin_field = 'mobile'
    get_mobile.allow_tags = True

    if 'fax' in settings.INSTALLED_APPS:
        def get_fax_icon(self, obj):
            if obj.fax:
                return """
                    <a href="%(reverse_url)s" title="Send Fax to %(fax)s">
                        <img src="%(url)sjmb/images/fax.png" alt="%(title)s" title="%(title)s"/>
                    </a>
                """ % {
                    'url': settings.MEDIA_URL,
                    'reverse_url': reverse('admin:fax_fax_add'),
                    'fax': obj.fax,
                    'title': obj.fax
                }
            return ""
        get_fax_icon.short_description = _("F")
    else:
        def get_fax_icon(self, obj):
            return obj.fax or ''
        get_fax_icon.short_description = _("Fax")
    get_fax_icon.order_admin_field = 'fax'
    get_fax_icon.allow_tags = True

    def get_user_icon(self, obj):
        if obj.user:
            # icona/link contatto con utente ATTIVO
            if obj.user.is_active:
                label = "%s: %s" % (obj.user.username, _('active'))
                return """
                    <a href="%(reverse_url)s" title="%(label)s">
                    <img src="%(url)sjmb/images/attivo.png" alt="%(title)s" />
                    </a>
                """ % {
                    "reverse_url": reverse('admin:auth_user_from_contact_change', args=[obj.user.id]),
                    "url": settings.STATIC_URL,
                    "title": _("active"),
                    "label": label,
                }
            # icona/link contatto con utente OBSOLETO
            else:
                label = "%s: %s" % (obj.user.username, _('inactive'))
                return """
                    <a href="%(reverse_url)s" title="%(label)s">
                    <img src="%(url)sjmb/images/sospeso.png" alt="%(title)s" />
                    </a>
                """ % {
                    "reverse_url": reverse('admin:auth_user_from_contact_change', args=[obj.user.id]),
                    "url": settings.STATIC_URL,
                    "title": _("inactive"),
                    "label": label,
                }
        # icona/link contatto SENZA utente
        else:
            return """
                <a href="%(reverse_url)s" alt="%(title)s">
                <img src="%(url)sjmb/images/inesistente.png" title="%(title)s" alt="%(title)s"/>
                </a>
            """ % {
                "reverse_url": reverse('admin:auth_user_from_contact_add', args=[obj.pk]),
                #                "reverse_url":  reverse('admin:auth_user_add'),
                "url":      settings.STATIC_URL,
                "title":    _("unconfigured"),
            }
    get_user_icon.short_description = _("Access")
    get_user_icon.allow_tags = True

    def get_changelist_form(self, request, **kwargs):
        return ContactAdminChangelistForm
#    list_editable = ('organization',)  # uncomment this to edit organization from the changelist.

    def get_form(self, request, obj=None, **kwargs):
        '''
        Return a form that forces all fields in the GET as not changeable
        '''
        # Alternative to declaring hidden_fields in the ConstraintForm
        hidden_fields = [key for key in request.GET.keys() if not key.startswith('_')]
        name = "%sForm" % self.model.__name__
        return type(smart_str(name), (ContactForm,), {
            'hidden_fields': hidden_fields,
        })


class ContactAdminChangelistForm(forms.ModelForm):
    organization = forms.ModelChoiceField(
        Organization.objects.all(),
        widget=al.ChoiceWidget('OrganizationAutocomplete',))

    class Meta:
        model = Contact
        exclude = []


###########################
# USER                    #
###########################
class UserContactAdminForm(UserCreationForm):

    def __init__(self, *args, **kw):
        super(UserCreationForm, self).__init__(*args, **kw)

    def save(self, commit=True):
        user = super(UserCreationForm, self).save(commit=False)
        user.set_password(self.cleaned_data["password1"])

        if getattr(self, 'contact_id', None):
            contact = Contact.objects.get(pk=self.contact_id)
            user.first_name = contact.first_name
            user.last_name = contact.last_name
            user.email = contact.email
            user.save()
            contact.user = user
            contact.save()
        else:
            user.save()
        return user


class UserAdmin(UserAdmin):

    list_display = (
        'username', 'email', 'first_name', 'last_name',
        'is_superuser', 'is_staff', 'is_active'
    )

    def get_urls(self):
        urls = super(UserAdmin, self).get_urls()
        my_urls = [
            # add from contact
            url(
                r'^from_contact/add/(?P<contact_id>\d+)/$',
                self.add_view,
                {'from_model': 'contact'},
                name='auth_user_from_contact_add'
            ),
            # change from contact
            url(
                r'^from_contact/change/(?P<object_id>\d+)/$',
                self.change_view,
                {'from_model': 'contact'},
                name='auth_user_from_contact_change'
            ),
            url(
                r'^from_contact/change/(\d+)/password/$',
                self.admin_site.admin_view(self.user_change_password),
                name='auth_user_from_contact_change_password'
            ),
            url(
                r'^from_contact/change/(\d+)/delete/$',
                self.delete_view,
                {'from_model': 'contact'},
                name='auth_user_from_contact_delete'
            )
        ]
        if django.VERSION[:2] < (1, 8):
            from django.conf.urls import patterns
            my_urls = patterns(*my_urls)
        return my_urls + urls

    def add_view(self, request, contact_id=None, form_url='', extra_context=None, from_model=None):
        self.from_model = from_model
        if self.from_model == 'contact':
            self.add_fieldsets = (
                (_('user information'), {'fields': (
                    'is_active',
                    'is_superuser',
                    'is_staff',
                    'username',
                    ('password1', 'password2',),
                    'groups'
                )}),
            )
            self.add_form = UserContactAdminForm
            self.add_form.contact_id = contact_id
        else:
            self.add_fieldsets = (
                (_('user information'), {'fields': (
                    'is_active',
                    'is_superuser',
                    'is_staff',
                    'username',
                    ('password1', 'password2',),
                    ('first_name', 'last_name',),
                    'email',
                    'groups'
                )}),
            )
            self.add_form = UserContactAdminForm

        response = super(UserAdmin, self).add_view(request, form_url, extra_context)
        if contact_id:
            response['location'] = reverse('admin:organization_contact_changelist')
        else:
            response['location'] = reverse('admin:auth_user_changelist')
        return response

    def change_view(self, request, object_id, form_url='', extra_context=None, from_model=None):
        self.from_model = from_model
        if from_model == 'contact':
            self.fieldsets = (
                (_('user information'), {'fields': (
                    'is_active',
                    'is_superuser',
                    'is_staff',
                    'username',
                    'password',
                    'groups',
                )}),
            )

        response = super(UserAdmin, self).change_view(request, object_id, form_url, extra_context)
        if from_model == 'contact':
            response['location'] = reverse('admin:organization_contact_changelist')
        return response

    def delete_view(self, request, object_id, extra_context=None, from_model=None):
        response = super(UserAdmin, self).delete_view(request, object_id, extra_context)
        if from_model == 'contact':
            response['location'] = reverse('admin:organization_contact_changelist')
        return response


if getattr(settings, 'AUTH_USER_MODEL', 'auth.User') == 'auth.User':
    pass
    admin.site.unregister(DjUser)
    admin.site.register(User, UserAdmin)

admin.site.register(Organization, OrganizationAdmin)
admin.site.register(OrganizationGroup, OrganizationGroupAdmin)
admin.site.register(OrganizationForm, OrganizationFormAdmin)
admin.site.register(OrganizationType, OrganizationTypeAdmin)
admin.site.register(Field, FieldAdmin)
admin.site.register(ContactRole, ContactRoleAdmin)
admin.site.register(ContactTitle, ContactTitleAdmin)
admin.site.register(Contact, ContactAdmin)

register_inline(ContactAjaxInline)
