# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login as auth_login
#from django.core.paginator import Paginator
from django.http import HttpResponseRedirect
from django.utils.translation import ugettext as _
from django.shortcuts import redirect

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool

from jmb.core.utils.functions import import_class
from jmb.core.utils.cmsutils import get_abs_url_from_slug, get_page_from_slug
from jmb.core.middleware.force_response import ForceResponse

from jmb.organization.cms.models import LoginSettings, RegistrationSettings
from jmb.organization.cms.models import RecoveryPasswordSettings, ResetPasswordSettings
from jmb.organization.utils import send_registration_email, send_activation_email, send_reset_password_email
from jmb.organization.utils import check_activation_url, check_reset_password_url


class LoginPlugin(CMSPluginBase):
    module = _('Organization')
    model = LoginSettings
    name = _('Login Form')
    admin_preview = False
    render_template = "cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = settings.ORGANIZATION_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        self.render_template = instance.form_template_name

        form_class = import_class(instance.form)
        # LOGIN

        if instance.recovery_password_slug:
            context.update({
                'recovery_url': get_abs_url_from_slug(pageslug=instance.recovery_password_slug)
            })

        if not user.is_authenticated():
            if request.method == 'POST' and 'login_submit' in request.POST:
                login_form = form_class(data=request.POST)
                if login_form.is_valid():
                    username = login_form.cleaned_data['username']
                    password = login_form.cleaned_data['password']
                    auth_user = authenticate(username=username, password=password)
                    if auth_user:
                        if auth_user.is_active:
                            auth_login(request, auth_user)
                            messages.info(request, _('welcome %s' % auth_user.first_name))
                            user = auth_user
                            raise ForceResponse(HttpResponseRedirect(request.get_full_path()))
                        else:
                            # Return a 'disabled account' error message
                            messages.error(request, _("disabled account"))
                            raise ForceResponse(HttpResponseRedirect(request.get_full_path()))
                    else:
                        # Return an 'invalid login' error message.
                        messages.error(request, _("invalid login"))
                        raise ForceResponse(HttpResponseRedirect(request.get_full_path()))

                else:
                    for error in login_form.errors.items():
                        try:
                            messages.error(request, str(error[1][0]))
                        except Exception as e:
                            messages.debug(request, _("error: %s" % e.message))
                    raise ForceResponse(HttpResponseRedirect(request.get_full_path()))

            else:
                login_form = form_class(request)

            context.update({
                'object': instance,
                'login_form': login_form,
                'user': user,
                'placeholder': placeholder,
            })

        return context

plugin_pool.register_plugin(LoginPlugin)


class RegistrationPlugin(CMSPluginBase):
    module = _('Organization')
    model = RegistrationSettings
    name = _('Registration Form')
    admin_preview = False
    render_template = "cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user

        self.render_template = settings.ORGANIZATION_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        if not user.is_anonymous():
            return context

        self.render_template = instance.form_template_name

        # attivazione utente
        if request.method == "GET":
            activated_user = check_activation_url(request)
            if activated_user:
                self.render_template = instance.activation_done_template
                send_activation_email(activated_user, instance, request)
                context.update({
                    'object': instance,
                    'placeholder': placeholder,
                })

                return context

        form_class = import_class(instance.form)
        registration_form = form_class()

        if (
            request.method == 'POST' and
            ('organization_registration_submit' in request.POST or 'contact_registration_submit' in request.POST)
        ):
            registration_form = form_class(data=request.POST, files=request.FILES)
            if registration_form.is_valid():
                # registrazione utente (e contatto)
                saved_obj = registration_form.save()
                try:
                    new_user = saved_obj.user
                except AttributeError:
                    new_user = None
                if new_user:
                    send_registration_email(new_user, instance, request)
                    # TODO
                    # iscrizione_a_newsletter(registration_form.cleaned_data, new_user)

                    # utilizzati messages invece che renderizzare nuovo template
                    messages.info(request, _('welcome %(u)s, an email as been sent to %(e)s to activate your account') % {
                        'u': new_user.first_name,
                        'e': new_user.email}
                    )

            else:
                for key, value in registration_form.errors.items():
                    error = "%s: %s" % (registration_form[key].label.capitalize(), value.as_text())
                    messages.error(request, error)
                raise ForceResponse(HttpResponseRedirect(request.get_full_path()))

        else:
            registration_form = form_class()

        context.update({
            'object': instance,
            'registration_form': registration_form,
            'placeholder': placeholder,
        })

        return context

plugin_pool.register_plugin(RegistrationPlugin)


class RecoveryPasswordPlugin(CMSPluginBase):
    module = _('Organization')
    model = RecoveryPasswordSettings
    name = _('Recovery Password')
    admin_preview = False
    render_template = "cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        request = context['request']
        user = request.user
        self.render_template = settings.ORGANIZATION_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        self.render_template = instance.form_template_name
        form_class = import_class(instance.form)
        if user.is_authenticated():
            redirect_url = get_abs_url_from_slug(pageslug=instance.password_reset_pageslug) or "/"
            raise ForceResponse(redirect(redirect_url))

        if request.method == 'POST' and 'recovery_password_submit' in request.POST:
            form = form_class(request.POST)
            if form.is_valid():
                try:
                    page_relative_url = get_abs_url_from_slug(
                        pageslug=instance.password_reset_pageslug) or "/"
                    form.save(instance, request, page_relative_url)
                    context['instance'] = instance
                    context['to_email'] = form.cleaned_data['email']
                    self.render_template = instance.done_template_name
                except Exception as e:
                    form = form_class()
        else:
            form = form_class()
        context.update({
            'instance': instance,
            'form': form,
            'fallback_page': get_page_from_slug(pageslug=instance.redirect_to)
        })
        return context

plugin_pool.register_plugin(RecoveryPasswordPlugin)


class ResetPasswordPlugin(CMSPluginBase):
    module = _('Organization')
    model = ResetPasswordSettings
    name = _('Reset Password')
    admin_preview = False
    render_template = "cms_plugins/empty.html"

    def render(self, context, instance, placeholder):
        """
        check the presence and validity of url of reset-password email message (based on
        django.contrib.auth.views.password_reset_confirm) for non-authenticated users.
        For authenticated users, get the usual change password form w/ old password.
        """
        request = context['request']
        user = request.user
        self.render_template = settings.ORGANIZATION_CMSPLUGIN_EMPTY_TEMPLATE
        if not instance.status:
            return context

        self.render_template = instance.form_template_name
        valid_url = False
        username = None
        form = None
        display_form = True
        fallback_page = None
        fallback_page_url = None

        if user.is_authenticated():
            # LOGGED-IN USERS
            # RESET PASSWORD SUCCESFULL
            if request.session and 'reset_pw_success' in request.session and request.session['reset_pw_success'] == True:
                # if password-reset process has been successfull, retrieve session data that confirm it, delete it from
                # session, set the done_template as render template in context and return it
                sess = request.session
                del request.session['reset_pw_success']
                if 'fallback_page' in request.session:
                    fallback_page = request.session['fallback_page']
                    del request.session['fallback_page']

                self.render_template = instance.done_template_name
                context.update({
                    'user': user,
                    'instance': instance,
                    'form': form,
                    'valid_url': valid_url,
                    'username': username,
                    'display_form': display_form,
                    'fallback_page': fallback_page,
                    #'fallback_page_url':fallback_page_url
                })
                return context
            else:
                form_class = import_class(instance.form_logged_in)
                form = form_class(user=request.user, data=request.POST or None)
                if request.method == "POST" and "reset_password_submit" in request.POST:
                    if form.is_valid():
                        # save the user instance w/ new password
                        user = form.save()
                        send_reset_password_email(
                            user=user, cms_model_instance=instance, request=request
                        )
                        # redirect sulla stessa pagina per comunicare avvenuto reset password
                        redirect_url = request.get_full_path()
                        request.session['reset_pw_success'] = True
                        request.session['fallback_page'] = get_page_from_slug(
                            pageslug=instance.redirect_to)
                        raise ForceResponse(redirect(redirect_url))
                context.update({
                    #'user':user,
                    'display_form': display_form,
                    'instance': instance,
                    'form': form,
                })
                return context
        else:
            # NOT LOGGED-IN USERS

            form_class = import_class(instance.form_notlogged_in)
            valid_url, _user = check_reset_password_url(request)
            if valid_url and _user:
                # valid URL
                if len(request.POST):
                    form = form_class(_user, request.POST)
                else:
                    form = form_class(_user)
                if form.is_valid():
                    # save the user instance w/ new password
                    user = form.save()
                    send_reset_password_email(
                        user=user, cms_model_instance=instance, request=request
                    )
                    user = authenticate(username=user.username,
                                        password=form.cleaned_data.get('new_password2'))
                    auth_login(request, user)

                    # redirect sulla stessa pagina come utente loggato per comunicare avvenuto
                    # reset password
                    redirect_url = request.get_full_path()
                    request.session['reset_pw_success'] = True
                    request.session['fallback_page'] = get_page_from_slug(
                        pageslug=instance.redirect_to)
                    raise ForceResponse(redirect(redirect_url))
                else:
                    # show change password form for not logged-in users
                    context.update({
                        'username': _user.username,
                        'display_form': True,
                        'instance': instance,
                        'form': form,
                    })
                    return context
            else:
                # URL not valid --> redirect to fallback page
                redirect_url = get_abs_url_from_slug(pageslug=instance.redirect_to) or "/"
                raise ForceResponse(redirect(redirect_url))


plugin_pool.register_plugin(ResetPasswordPlugin)
